#ifndef _ActivCalibrationDefined
#define _ActivCalibrationDefined

/*
 * WARNING: this means that 64-bit calib data and 32-bit calib data will be different sizes.
 * Linux has long has 64-bit, not 32-bit.
 * It isn't a problem because we compile everything native on Linux, unlike Windows.
 * */
#define SLONG long
#ifdef WIN32

typedef int int32_t;
typedef long long int64_t;

#else
#ifdef LINUX_VERSION_CODE

#include <linux/types.h>

#else

#include <stdint.h>

#endif

#endif

typedef struct _CALIB_DATA
{
	// it would be better to use X,Y structs, but we need
	// to keep mathrout.cpp as it is (it is shared with
	// other projects)
	SLONG m_bBox[4];			// bbox is screen bounding rect in digitizer space, radix 0.
	SLONG m_rbBox;			// maximum radix of bbox values without overflow.
	SLONG m_mNorm[4][4];		// mNorm is normalization matrix.
	SLONG m_rNorm;			// rNorm is radix of mNorm.
	SLONG m_accy;			// accuracy limit for matrix-mul output, radix rNorm.
	SLONG m_Outrect[4];		// outrect is desired output rect, radix 0.
	SLONG m_rOutrect;
	//LONG m_PrevOut[2];

	SLONG m_wndrect[4];
#ifndef WIN32
    SLONG m_wndrect_normalised[4];
#endif
	SLONG m_desktoprect[4];
	SLONG m_x,m_y;
} CALIB_DATA, *PCALIB_DATA;;

typedef struct _CALIB_DATA32
{
	// it would be better to use X,Y structs, but we need
	// to keep mathrout.cpp as it is (it is shared with
	// other projects)
	int m_bBox[4];			// bbox is screen bounding rect in digitizer space, radix 0.
	int m_rbBox;			// maximum radix of bbox values without overflow.
	int m_mNorm[4][4];		// mNorm is normalization matrix.
	int m_rNorm;			// rNorm is radix of mNorm.
	int m_accy;			// accuracy limit for matrix-mul output, radix rNorm.
	int m_Outrect[4];		// outrect is desired output rect, radix 0.
	int m_rOutrect;
	//LONG m_PrevOut[2];

	int m_wndrect[4];
	int m_wndrect_normalised[4];
	int m_desktoprect[4];
	int m_x,m_y;
} CALIB_DATA32, *PCALIB_DATA32;

typedef struct _calibration_point
{
    int32_t x, y;
} calibration_point;

typedef struct _calibration_rect {
    int32_t left, top;
    int32_t right, bottom;
} calibration_rect;

typedef struct _calibration_matrix {
    int64_t m11, m12, m13;
    int64_t m21, m22, m23;
    int64_t dx,dy;
    int64_t m33;
} calibration_matrix;

typedef struct _calibration_quad_region {
    //! coordinates of each rectangle on the desktop. Initialised by the calibration program.
    calibration_rect display_rect;

    //! raw coordinates of four points clicked. the 5th point is a copy of the first. This is not rectangular.
    calibration_point raw_points[5];

    //! raw coordinates of mid point.
    calibration_point raw_min;

    //! raw coordinates of rectangle contained within the 4 raw points
    calibration_rect hit_rect;

    //! desktop size
    calibration_point desktop_size;

    //! screen coordinates. On Windows, relative to leftmost being 0,0; otherwise, relative to primary screen.
    calibration_rect screen_rect;

    //! Offset from topmost. Used on Mac to render private coords to the whole desktop.
    calibration_point normalise_offset;

    //! matrix generated by transformToRect. Generated as double in the calibration, the cast down to long and passed to driver.
    calibration_matrix matrix;
} calibration_quad_region;

typedef struct _TOUCH_PACKET
{
    struct {
        unsigned char ID;
        unsigned short x,y,radius;
    } data[12];
} TOUCH_PACKET;

static const long long SCALE_FACTOR[2] = { 1 << 30, 1 << 12};

#endif
